@@ -1,7 +1,7 @@
 #include <RcppEigen.h>
 #include "bvhardraw.h"
-#include <progress.hpp>
-#include <progress_bar.hpp>
+#include "bvharprogress.h"
+#include "bvharinterrupt.h"
 
 //' VAR-SV by Gibbs Sampler
 //' 
@@ -107,7 +107,8 @@ Rcpp::List estimate_var_sv(int num_iter, int num_burn,
   Eigen::MatrixXd contem_coef_record = Eigen::MatrixXd::Zero(num_iter + 1, num_lowerchol); // a = a21, a31, a32, ..., ak1, ..., ak(k-1)
   Eigen::MatrixXd lvol_sig_record = Eigen::MatrixXd::Zero(num_iter + 1, dim); // sigma_h^2 = (sigma_(h1i)^2, ..., sigma_(hki)^2)
   Eigen::MatrixXd lvol_init_record = Eigen::MatrixXd::Zero(num_iter + 1, dim); // h0 = h10, ..., hk0
-  Eigen::MatrixXd lvol_record = Eigen::MatrixXd::Zero(num_design * (num_iter + 1), dim); // time-varying h = (h_1, ..., h_k) with h_j = (h_j1, ..., h_jn): h_ij in each dim-block
+  // Eigen::MatrixXd lvol_record = Eigen::MatrixXd::Zero(num_design * (num_iter + 1), dim); // time-varying h = (h_1, ..., h_k) with h_j = (h_j1, ..., h_jn): h_ij in each dim-block
+	Eigen::MatrixXd lvol_record = Eigen::MatrixXd::Zero(num_iter + 1, num_design * dim); // time-varying h = (h_1, ..., h_k) with h_j = (h_j1, ..., h_jn), row-binded
   // SSVS--------------
   Eigen::MatrixXd coef_dummy_record(num_iter + 1, num_alpha);
   Eigen::MatrixXd coef_weight_record(num_iter + 1, num_grp);
@@ -122,7 +123,8 @@ Rcpp::List estimate_var_sv(int num_iter, int num_burn,
   coef_record.row(0) = coefvec_ols;
   contem_coef_record.row(0) = Eigen::VectorXd::Zero(num_lowerchol); // initialize a as 0
   lvol_init_record.row(0) = (y - x * coef_mat).transpose().array().square().rowwise().mean().log(); // initialize h0 as mean of log((y - x alpha)^T (y - x alpha))
-  lvol_record.block(0, 0, num_design, dim) = lvol_init_record.row(0).replicate(num_design, 1);
+  // lvol_record.block(0, 0, num_design, dim) = lvol_init_record.row(0).replicate(num_design, 1);
+	Eigen::MatrixXd lvol_draw = lvol_init_record.row(0).replicate(num_design, 1); // h_j = (h_j1, ..., h_jn) for MCMC update
   lvol_sig_record.row(0) = .1 * Eigen::VectorXd::Ones(dim);
   // SSVS--------------
   coef_dummy_record.row(0) = Eigen::VectorXd::Ones(num_alpha);
@@ -136,14 +138,14 @@ Rcpp::List estimate_var_sv(int num_iter, int num_burn,
   Eigen::MatrixXd chol_lower = Eigen::MatrixXd::Zero(dim, dim); // L in Sig_t^(-1) = L D_t^(-1) LT
   Eigen::MatrixXd latent_innov(num_design, dim); // Z0 = Y0 - X0 A = (eps_p+1, eps_p+2, ..., eps_n+p)^T
   Eigen::MatrixXd ortho_latent(num_design, dim); // orthogonalized Z0
-  Eigen::MatrixXd lvol_draw = lvol_record.block(0, 0, num_design, dim);
+  // Eigen::MatrixXd lvol_draw = lvol_record.block(0, 0, num_design, dim);
   // Corrected triangular factorization-------
   Eigen::VectorXd prior_mean_j = Eigen::VectorXd::Zero(dim_design); // Prior mean vector of j-th column of A
   Eigen::MatrixXd prior_prec_j = Eigen::MatrixXd::Identity(dim_design, dim_design); // Prior precision of j-th column of A
   Eigen::MatrixXd coef_j = coef_mat; // j-th column of A = 0: A(-j) = (alpha_1, ..., alpha_(j-1), 0, alpha_(j), ..., alpha_k)
   coef_j.col(0) = Eigen::VectorXd::Zero(dim_design);
   Eigen::VectorXd response_contem(num_design); // j-th column of Z0 = Y0 - X0 * A: n-dim
-  Eigen::MatrixXd sqrt_sv(num_design, dim); // stack sqrt of exp(h_t) = (exp(-h_1t / 2), ..., exp(-h_kt / 2)), t = 1, ..., n => n x k
+  Eigen::MatrixXd sqrt_sv = (-lvol_draw / 2).array().exp(); // stack sqrt of exp(h_t) = (exp(-h_1t / 2), ..., exp(-h_kt / 2)), t = 1, ..., n => n x k
   int contem_id = 0;
   // SSVS--------------
   Eigen::VectorXd prior_sd(num_coef);
@@ -161,9 +163,10 @@ Rcpp::List estimate_var_sv(int num_iter, int num_burn,
   Eigen::MatrixXd global_shrinkage_mat = Eigen::MatrixXd::Zero(dim_design, dim);
   Eigen::VectorXd grp_vec = vectorize_eigen(grp_mat);
   // Start Gibbs sampling-----------------------------------
-  Progress p(num_iter, display_progress);
+	bvharprogress bar(num_iter, display_progress);
+	bvharinterrupt();
   for (int i = 1; i < num_iter + 1; i ++) {
-    if (Progress::check_abort()) {
+		if (bvharinterrupt::is_interrupted()) {
       if (prior_type == 2) {
         return Rcpp::List::create(
           Rcpp::Named("alpha_record") = coef_record,
@@ -193,7 +196,10 @@ Rcpp::List estimate_var_sv(int num_iter, int num_burn,
         Rcpp::Named("sigh_record") = lvol_sig_record
       );
     }
-    p.increment();
+		bar.increment();
+		if (display_progress) {
+			bar.update();
+		}
     // 1. alpha----------------------------
     chol_lower = build_inv_lower(dim, contem_coef_record.row(i - 1));
     switch(prior_type) {
@@ -228,7 +234,6 @@ Rcpp::List estimate_var_sv(int num_iter, int num_burn,
       shrink_record.row(i - 1) = (Eigen::MatrixXd::Identity(num_coef, num_coef) + prior_alpha_prec).inverse().diagonal();
       break;
     }
-    sqrt_sv = (-lvol_draw / 2).array().exp(); // n x k
     for (int j = 0; j < dim; j++) {
       prior_mean_j = prior_alpha_mean.segment(dim_design * j, dim_design);
       prior_prec_j = prior_alpha_prec.block(dim_design * j, dim_design * j, dim_design, dim_design);
@@ -305,8 +310,10 @@ Rcpp::List estimate_var_sv(int num_iter, int num_burn,
         ortho_latent.col(t)
       );
     }
-    lvol_record.block(num_design * i, 0, num_design, dim) = lvol_draw;
+    // lvol_record.block(num_design * i, 0, num_design, dim) = lvol_draw;
+		lvol_record.row(i) = vectorize_eigen(lvol_draw.transpose());
     // 3. a---------------------------------
+    sqrt_sv = (-lvol_draw / 2).array().exp(); // n x k
     switch (prior_type) {
     case 2:
       // SSVS
@@ -372,7 +379,7 @@ Rcpp::List estimate_var_sv(int num_iter, int num_burn,
   if (prior_type == 2) {
     return Rcpp::List::create(
       Rcpp::Named("alpha_record") = coef_record.bottomRows(num_iter - num_burn),
-      Rcpp::Named("h_record") = lvol_record,
+      Rcpp::Named("h_record") = lvol_record.bottomRows(num_iter - num_burn),
       Rcpp::Named("a_record") = contem_coef_record.bottomRows(num_iter - num_burn),
       Rcpp::Named("h0_record") = lvol_init_record.bottomRows(num_iter - num_burn),
       Rcpp::Named("sigh_record") = lvol_sig_record.bottomRows(num_iter - num_burn),
@@ -382,7 +389,7 @@ Rcpp::List estimate_var_sv(int num_iter, int num_burn,
     shrink_record.row(num_iter) = (Eigen::MatrixXd::Identity(num_coef, num_coef) + prior_alpha_prec).inverse().diagonal();
     return Rcpp::List::create(
       Rcpp::Named("alpha_record") = coef_record.bottomRows(num_iter - num_burn),
-      Rcpp::Named("h_record") = lvol_record,
+      Rcpp::Named("h_record") = lvol_record.bottomRows(num_iter - num_burn),
       Rcpp::Named("a_record") = contem_coef_record.bottomRows(num_iter - num_burn),
       Rcpp::Named("h0_record") = lvol_init_record.bottomRows(num_iter - num_burn),
       Rcpp::Named("sigh_record") = lvol_sig_record.bottomRows(num_iter - num_burn),
@@ -393,7 +400,7 @@ Rcpp::List estimate_var_sv(int num_iter, int num_burn,
   }
   return Rcpp::List::create(
     Rcpp::Named("alpha_record") = coef_record.bottomRows(num_iter - num_burn),
-    Rcpp::Named("h_record") = lvol_record,
+    Rcpp::Named("h_record") = lvol_record.bottomRows(num_iter - num_burn),
     Rcpp::Named("a_record") = contem_coef_record.bottomRows(num_iter - num_burn),
     Rcpp::Named("h0_record") = lvol_init_record.bottomRows(num_iter - num_burn),
     Rcpp::Named("sigh_record") = lvol_sig_record.bottomRows(num_iter - num_burn)
