@@ -33,4 +33,22 @@ test_that("Test for vharlse class", {
     num_col
   )
 })
+
+test_that("Computation Methods", {
+  skip_on_cran()
+  
+  fit_test_nor <- vhar_lm(etf_vix[, 1:3])
+  fit_test_llt <- vhar_lm(etf_vix[, 1:3], method = "chol")
+  fit_test_qr <- vhar_lm(etf_vix[, 1:3], method = "qr")
+
+  expect_equal(
+    fit_test_nor$coefficients,
+    fit_test_llt$coefficients
+  )
+
+  expect_equal(
+    fit_test_nor$coefficients,
+    fit_test_qr$coefficients
+  )
+})
 #> Test passed 🌈
\ No newline at end of file
